/* eslint-disable no-magic-numbers */
import { PIN_SIZE } from '@/constants/canvas';
import { allBioEntitesSelectorOfCurrentMap } from '@/redux/bioEntity/bioEntity.selectors';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import { Feature } from 'ol';
import { Point } from 'ol/geom';
import BaseLayer from 'ol/layer/Base';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import Icon from 'ol/style/Icon';
import Style from 'ol/style/Style';
import { useMemo } from 'react';
import { useSelector } from 'react-redux';
import { getCanvasIcon } from './getCanvasIcon';

export const useOlMapPinsLayer = (): BaseLayer => {
  const pointToProjection = usePointToProjection();
  const bioEntites = useSelector(allBioEntitesSelectorOfCurrentMap);

  const bioEntityFeatures = useMemo(
    () =>
      bioEntites.map(({ bioEntity: { x, y, name, width, height } }, index) => {
        const point = {
          x: x + width / 2,
          y: y + height / 2,
        };

        const feature = new Feature({
          geometry: new Point(pointToProjection(point)),
          name,
        });

        const style = new Style({
          image: new Icon({
            displacement: [0, PIN_SIZE.height],
            anchorXUnits: 'fraction',
            anchorYUnits: 'pixels',
            img: getCanvasIcon({
              color: '#106AD7',
              value: index + 1,
            }),
          }),
        });

        feature.setStyle(style);

        return feature;
      }),
    [bioEntites, pointToProjection],
  );

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features: [...bioEntityFeatures],
    });
  }, [bioEntityFeatures]);

  const pinsLayer = useMemo(
    () =>
      new VectorLayer({
        source: vectorSource,
      }),
    [vectorSource],
  );

  return pinsLayer;
};
